import json
import requests
from datetime import datetime, timezone, timedelta

def print_dict(dict, sort=True):
    print(json.dumps(dict, sort_keys=sort, indent=4))

    return

def print_dict_to_file(dict, filename, sort=True):
    with open(filename, 'w') as f:
      json.dump(dict, f, indent=4, sort_keys=sort)

    return


def make_resource_signature(resource_identifiers):

    signature_unencoded = ""
    for identifier in resource_identifiers:
        signature = signature + identifier['type'] + ':' + identifier['value'] + ','

    signature = signature_unencoded.encode(encoding='UTF-8', errors='strict')

    return signature


def decode_resource_signature(signature):

    assert(type(signature) == "string")
    for identifier in resource_identifiers:
        signature = signature + identifier['type'] + ':' + identifier['value'] + ','

    signature = signature_unencoded.encode(encoding='UTF-8', errors='strict')

    return signature


def make_resources_uri(api_uri, client_id):
    resources_uri = api_uri + '/tenants/' + client_id + '/resources'

    return resources_uri

def get_token(auth_uri, key, secret):
    # Get OAuth2 token

    headers = {"Content-Type": "application/x-www-form-urlencoded", "Accept": "application/json"}
    data = { "grant_type": "client_credentials", "client_id": key, "client_secret": secret }


    r = requests.post(auth_uri, data=data, headers=headers)

    json_response = r.json()
    # print (json.dumps(json_response, indent=2))

    auth = str(json_response["token_type"]) + " " + str(json_response["access_token"])
    access_header = {"Content-Type": "application/json", "Accept": "application/json", "Authorization": auth}

    return access_header


def print_api_setup(access_header, uri):
    print("access header: " + json.dumps(access_header, indent=4))
    print("url: " + uri)

    return


def get_datetime(date_string):
    dt = datetime.fromisoformat(date_string)

    return dt

def get_epoch_time(dt):
    et = int(round(dt.timestamp()))

    return et

def get_timedelta(days=0, hours=0, minutes=0, seconds=0, microseconds=0, milliseconds=0, weeks=0):
    td = timedelta(days=days, hours=hours, minutes=minutes, seconds=seconds,
                   microseconds=microseconds, milliseconds=microseconds, weeks=weeks)

    return td


def search_resources(access_header, api_uri, client_id):

    resources = []

    # list resources in a client
    resource_uri = make_resources_uri(api_uri, client_id)
    query_string = "?pageNo="

    #print("searching for resources in client:{}".format(client_id))

    results = []
    ids = []

    pageNo = 1
    nextPage = True

    while (nextPage):

        uri = resource_uri + "/search" + query_string + str(pageNo)
        # print("uri: " + uri)

        try:
            r = requests.get(uri, headers=access_header)
        except:
            raise Exception("API call unsuccessful")

        response = r.json()

        # print_dict(response)

        results = results + response["results"]

        pageNo = pageNo + 1
        uri = uri + query_string + str(pageNo)
        nextPage = response["nextPage"]

        # print("totalResults: " + str(response["totalResults"]))
        # print("page: " + str(pageNo))

    assert (len(results) == response["totalResults"]), "Total results and retrieved results do not match"

    for resource in results:
        resources.append(resource)


    return resources


def get_resource_details(access_header, base_uri, resource_ids):
    # get details on each resource
    # returns array of dictionary - each array element is resource details for a resource

    resource_details = []

    for client_id in resource_ids.keys():

        print(
            "getting resource details for client:{} with {} resources".format(client_id, len(resource_ids[client_id])))

        resource_uri = make_resources_uri(base_uri, client_id)

        res_count = 1
        for res in resource_ids[client_id]:
            uri = resource_uri + "/" + res

            print("resource count: " + str(res_count))
            res_count = res_count + 1

            try:
                r = requests.get(uri, headers=access_header)
            except:
                raise Exception("API call unsuccessful")

            response = r.json()

            resource_details.append(response)

    return resource_details


def create_resources(access_header, api_uri, client_id, resource_list):
    # Create a resource
    uri = make_resources_uri(api_uri, client_id)

    for res in resource_list:
        try:
            r = requests.post(uri, data=json.dumps(res), headers=access_header)
        except:
            raise Exception("API call unsuccessful")


    return


def get_metrics_on_resource(access_header, api_uri, client_id, resource_id):

    #https://{api-url}/api/v2/metric/tenants/client_7/rtypes/DEVICE/resources/725ab798-e9b1-474c-aa03-de9fb6a6e0c4/metrics
    uri = api_uri + '/metric/tenants/'+client_id+'/rtypes/DEVICE/resources/'+resource_id

    # print_api_setup(access_header, uri)

    try:
        r = requests.get(uri, headers=access_header)
    except:
        raise Exception("API call unsuccessful")

    response = r.json()

    return response


def get_metric_values_on_resource(access_header, api_uri, client_id, resource_id, metricName, startTime, endTime):
    # https://{api-url}/api/v2/metric/search?tenant=client_20&rtype=DEVICE&resource=ab342123-5bfb-434c-aae4-03611ca020d9&metric=system.cpu.utilization&startTime=1536643494&endTime=1536661494

    uri = api_uri + '/metric/search?tenant=' + client_id +'&rtype=DEVICE'+'&resource='+ resource_id
    uri = uri + '&metric=' + metricName
    uri = uri + '&startTime=' + startTime + '&endTime=' + endTime

    # print_api_setup(access_header, uri)

    try:
        r = requests.get(uri, headers=access_header)
    except:
        raise Exception("API call unsuccessful")

    response = r.json()

    return response


def post_metric_values_on_resource(access_header, api_uri, client_id, resource_id, metric_values):
    # get metrics monitored on a resource

    # https://{api-url}/api/v2/metric/tenants/client_7/rtypes/DEVICE/resources/735ab798-e9b4-474c-aa03-de9fb6a6e0f5/metrics
    # "/api/v2/metric/tenants/{CLIENT_TOKEN}/rtypes/DEVICE/resources/{RESOURCE_UUID}/metrics"
    uri = api_uri+'/metric/tenants/'+client_id+'/rtypes/DEVICE/resources/'+ resource_id+'/metrics'

    #print_api_setup(access_header, uri)
    #print_dict(metric_values)

    try:
        r = requests.post(uri, data=json.dumps(metric_values), headers=access_header)
        # r = "{abc:123}"
    except:
        raise Exception("API call unsuccessful")

    print ('post metric status: ', r.status_code)
    #return r.json()
    return






def dump_resources(client_ids, resources):

    resourceDump = {
        "numClients" : len(client_ids),
        "clients" : client_ids,
        "numResources": len(resources),
        "resources" : resources
    }

    return resourceDump

def dump_metrics(client_id, resource_id, metrics):

    metricDump = {
        "client" : client_id,
        "resource": resource_id,
        "numMetrics": len(metrics),
        "metrics" : metrics
    }

    return metricDump

def dump_metric_values(client_id, resource_id, startTime, endTime, metric_values):

    metricValueDump = {
        "client" : client_id,
        "resource": resource_id,
        "startTime" : startTime,
        "endTime" : endTime,
        "metrics" : metric_values
    }

    return metricValueDump
